#!/bin/sh

### This script returns :
###   0: On success.
###   from 1 to ??(under 64): Error has been occured, but Flash memory has NOT YET been changed.
###   from 64 to ???(under 128): Error has been returned by external shell program, but Flash memory has NOT YET been changed.

TMP_LOG_FILE=/mnt/data/log/check_after_extract_log.txt
rm -f "${TMP_LOG_FILE}"
{
#--- for DeBuG only ---
##set -x
#--- for DeBuG only ---

echo "Seconds:${SECONDS}"

MYSELF_DIR=$(dirname "${0}")
ENVIRONMENT_FILE="${MYSELF_DIR}/4gpa_environment.rc"

if [ ! -f "${ENVIRONMENT_FILE}" ]; then
  echo "Error: ${ENVIRONMENT_FILE} is not found!!!" 1>&2
  exit 1
fi

## Include environment file.
. "${ENVIRONMENT_FILE}"

if [ ! -d ${FW_DIR} ]; then
  echo "Error: ${FW_DIR} is not found!!!" 1>&2
  exit 2
fi

RESULT_NG=0

## --------------------------------------------------------
## MODEL check (copied from 4gpa_installer.sh)
ChkModelQorM () {
  QorM=`cat /home/root/tmpconf/model_info.csv | cut -d',' -f 3`
  
  echo $QorM
}

model=`ChkModelQorM`
if [ X"$model" = X"M" ]; then
  echo "check_after_extract ChkModelQorM: M"
  FW_DTB=${FW_DTB_M}
  STATUS_MTD_KERNEL_CHKSUM=${STATUS_MTD_KERNEL_CHKSUM_M}
  DTB_BLOCKS_MD5SUM=${DTB_BLOCKS_MD5SUM_M}
else
  echo "check_after_extract ChkModelQorM: not M (Q)"
  FW_DTB=${FW_DTB_Q}
  STATUS_MTD_KERNEL_CHKSUM=${STATUS_MTD_KERNEL_CHKSUM_Q}
  DTB_BLOCKS_MD5SUM=${DTB_BLOCKS_MD5SUM_Q}
fi

## --------------------------------------------------------
Chkfile () {
    ## Initialized by error value
    RESULT_NG=1

    DIR_FOR_CHECK=${1}
    DAT_FILE=${2}
    RESULT_FILE=/tmp/chkfile_result.${$}
    rm -f ${RESULT_FILE}

    NUM_OF_FILE=$(cat ${DAT_FILE} | wc -l)
    (cd ${DIR_FOR_CHECK} && sudo md5sum --check --strict ${DAT_FILE}) > ${RESULT_FILE} 2>/dev/null
    NUM_OF_OK=$(egrep ': OK$' ${RESULT_FILE} | wc -l)

    if [ ${NUM_OF_FILE} -eq ${NUM_OF_OK} ]; then
        RESULT_NG=0
        echo "Success: check file contents of ${DIR_FOR_CHECK}"
    else
        echo "Failed: check file contents of ${DIR_FOR_CHECK}"
        echo "        NG line(s) below"
        echo '******************************************************'
        egrep -v ': OK$' ${RESULT_FILE}
        echo '******************************************************'
    fi

    rm -f ${RESULT_FILE}
}
## --------------------------------------------------------

if [ ! -f ${INSTALLER_SH} ]; then
  echo "Error: ${INSTALLER_SH} is not found!!!" 1>&2
  exit 3
fi

if [ ! -f ${FW_KERNEL} ]; then
  echo "Error: ${FW_KERNEL} is not found!!!" 1>&2
  exit 4
fi

if [ ! -f ${FW_DTB} ]; then
  echo "Error: ${FW_DTB} is not found!!!" 1>&2
  exit 5
fi

if [ ! -f ${FW_UPDATE_UBI} ]; then
  echo "Error: ${FW_UPDATE_UBI} is not found!!!" 1>&2
  exit 6
fi

if [ ! -f ${STATUS_MTD_FILE_FWUPDATING} ]; then
  echo "Error: ${STATUS_MTD_FILE_FWUPDATING} is not found!!!" 1>&2
  exit 7
fi

if [ ! -f ${STATUS_MTD_FILE_NORMAL} ]; then
  echo "Error: ${STATUS_MTD_FILE_NORMAL} is not found!!!" 1>&2
  exit 8
fi

if [ ! -f ${STATUS_MTD_KERNEL_CHKSUM} ]; then
  echo "Error: ${STATUS_MTD_KERNEL_CHKSUM} is not found!!!" 1>&2
  exit 9
fi

##### if [ ! -f ${DSP_UPDATER_FILE} ]; then
#####   echo "Error: ${DSP_UPDATER_FILE} is not found!!!" 1>&2
#####   exit 10
##### fi

##### if [ ! -f ${DSP_CHKSUM_FILE} ]; then
#####   echo "Error: ${DSP_CHKSUM_FILE} is not found!!!" 1>&2
#####   exit 11
##### fi

if [ ! -f ${FIRMWARE_BIN_MD5SUM} ]; then
  echo "Error: ${FIRMWARE_BIN_MD5SUM} is not found!!!" 1>&2
  exit 12
fi

if [ ! -f ${KERNEL_BLOCKS_MD5SUM} ]; then
  echo "Error: ${KERNEL_BLOCKS_MD5SUM} is not found!!!" 1>&2
  exit 13
fi

if [ ! -f ${DTB_BLOCKS_MD5SUM} ]; then
  echo "Error: ${KERNEL_BLOCKS_MD5SUM} is not found!!!" 1>&2
  exit 14
fi

## Check md5sum for each file under ${FW_DIR}
##echo "Seconds:${SECONDS}"
Chkfile "${FW_DIR}" "${FIRMWARE_BIN_MD5SUM}"
  if [ ${RESULT_NG} -ne 0 ]; then exit 14; fi
##echo "Seconds:${SECONDS}"

echo "Seconds:${SECONDS}"

} 2>&1 | tee ${TMP_LOG_FILE}
sync; sync; sync

#--- for DeBuG only ---
##sudo cp ${TMP_LOG_FILE} /dev/console
##sleep 5 ## because /dev/console is very slow.
#--- for DeBuG only ---

exit 0
