#!/bin/sh

### This script returns :
###   0: On success.
###   from 1 to ??(under 64): Error has been occured, and Flash memory has ALREADY been changed.
###   from 64 to ???(under 128): Error has been returned by external shell program, and Flash memory has ALREADY been changed.
CANCEL_NEW_PLAY="/tmp/cancel-new-play.txt"
TMP_LOG_FILE=/mnt/data/log/installer_log.txt
sudo rm -f "${TMP_LOG_FILE}"
{
#--- for DeBuG only ---
##set -x
#--- for DeBuG only ---

echo "Seconds:${SECONDS}"

MYSELF_DIR=$(dirname "${0}")
ENVIRONMENT_FILE="${MYSELF_DIR}/4gpa_environment.rc"

## Include environment file.
. "${ENVIRONMENT_FILE}"

RESULT_NG=0

## --------------------------------------------------------
## MODEL check
ChkModelQorM () {
  QorM=`cat /home/root/tmpconf/model_info.csv | cut -d',' -f 3`
  
  echo $QorM
}

model=`ChkModelQorM`
if [ X"$model" = X"M" ]; then
  echo "check_after_extract ChkModelQorM: M"
  FW_DTB=${FW_DTB_M}
  STATUS_MTD_KERNEL_CHKSUM=${STATUS_MTD_KERNEL_CHKSUM_M}
  DTB_BLOCKS_MD5SUM=${DTB_BLOCKS_MD5SUM_M}
else
  echo "check_after_extract ChkModelQorM: not M (Q)"
  FW_DTB=${FW_DTB_Q}
  STATUS_MTD_KERNEL_CHKSUM=${STATUS_MTD_KERNEL_CHKSUM_Q}
  DTB_BLOCKS_MD5SUM=${DTB_BLOCKS_MD5SUM_Q}
fi
## --------------------------------------------------------
ChkRecovDtbChksum () {
    ## Initialized by error value
    RESULT_NG=1

    DEVICE="${1}"
    DUMP_FILE=/tmp/chkrecovdtb_dump.${$}
    sudo rm -f ${DUMP_FILE}

    BLOCK_SZ=$((128*1024))
    FILE_SZ_DTB=$(wc -c "${FW_DTB}" | cut -d' ' -f 1)
    BLKLEN_DTB=$(( ( ( ${FILE_SZ_DTB} + ${BLOCK_SZ} - 1 ) / ${BLOCK_SZ} ) ))
    TOREAD_LEN_DTB=$(( ${BLKLEN_DTB} * ${BLOCK_SZ} ))
    OFFSET_OF_DTB_IN_KERNEL_PARTITION="0x720000"
    
    sudo nanddump --bb=skipbad -f "${DUMP_FILE}" "${DEVICE}" -s ${OFFSET_OF_DTB_IN_KERNEL_PARTITION} -l ${TOREAD_LEN_DTB}
    if [ ${?} -ne 0 ]; then
        echo "Failed: dump dtb in ${DEVICE}"
    else
        MD5SUM_RESULT=$(md5sum -b "${DUMP_FILE}" | cut -d' ' -f 1)
        MD5SUM_EXPECTED=$(cat "${DTB_BLOCKS_MD5SUM}")
        if [ "X${MD5SUM_RESULT}" != "X${MD5SUM_EXPECTED}" ]; then
            echo "Failed: md5sum of dump dtb in ${DEVICE} does not match expected(${MD5SUM_EXPECTED}) vs result(${MD5SUM_RESULT})"
        else
            RESULT_NG=0
            echo "Success: md5sum of dump ${DEVICE} matches"
        fi
    fi
    sudo rm -f ${DUMP_FILE}
}
## --------------------------------------------------------
## Freq change & Recovery dtb update
UpdateRecovDTB () {
  isPreChkDTB=${1}
  isUpdate="true"

  ## Check booting up mode.
  STATUS_MTD=$(cat /proc/cmdline | sed -ne 's/^.*status_mtd=\([^ :]*\).*$/\1/p')
  if [ X${STATUS_MTD} = X"normal" ]; then
    echo "Current mode: Normal"

    echo "isPreChkDTB: $isPreChkDTB"
    if [ X$isPreChkDTB = X"true" ]; then
      # Pre-check of recov dtb checksum
      ChkRecovDtbChksum /dev/mtd5
      if [ ${RESULT_NG} -ne 0 ]; then 
        echo "Precheck ChkRecovDtbChksum /dev/mtd5 NG. Recovery DTB will be updated."
        isUpdate="true"
      else
        echo "Precheck ChkRecovDtbChksum /dev/mtd5 OK. Recovery DTB will not be updated."
        isUpdate="false"
      fi
    fi

    if [ X$isUpdate = X"true" ]; then
      echo "Recovery DTB update: flash_erase /dev/mtd5 0x720000 0"
      sudo flash_erase /dev/mtd5 0x720000 0
      if [ ${?} -ne 0 ]; then 
        echo "fail to flash_erase /dev/mtd5 0x720000 0"
        exit 1;
      fi
      echo "Recovery DTB update: nandwrite -p /dev/mtd5 -s 0x720000 ${FW_DTB}"
      sudo nandwrite -p /dev/mtd5 -s 0x720000 ${FW_DTB}
      if [ ${?} -ne 0 ]; then 
        echo "fail to nandwrite -p /dev/mtd5 -s 0x720000 ${FW_DTB}"
        exit 1; 
      fi

      ChkRecovDtbChksum /dev/mtd5
      if [ ${RESULT_NG} -ne 0 ]; then 
        echo "ChkRecovDtbChksum /dev/mtd5 NG."
        exit 1
      fi
      echo "ChkRecovDtbChksum /dev/mtd5 OK."

      # Make a log file to know the recovery version because we can not update recovery version file.
      touch /mnt/data/log/CPU900MHz_Voltage_Updated.txt
    fi
  else 
    echo "Current mode: Recovery. Recovery DTB will not be updated."
  fi
}

#--- for IPSP-949 CPU 900MHz Voltage Problem---Start
echo '/usr/bin/cpufreq-info -f'
/usr/bin/cpufreq-info -f

echo cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq
cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq

echo 'od -td4 --endian=big /proc/device-tree/cpus/cpu\@0/fsl\,soc-operating-points'
od -td4 --endian=big /proc/device-tree/cpus/cpu\@0/fsl\,soc-operating-points

${MYSELF_DIR}/cpufreq
if [ $? -ne 0 ]; then
  sync; sync; sync
  echo fail to change freq
  echo abort to update firmware
  sudo /sbin/reboot
  exit 1
fi

echo 'cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq'
cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq

echo '/usr/bin/cpufreq-info -f'
/usr/bin/cpufreq-info -f

# 900MHz voltage flag (true=OK(Correct), false=NG(Wrong))
flagVoltageOK="false"

# Recovery DTB update
## If correct voltage is not set, recovery dtb will be updated
voltage_900m=`od -td4 --endian=big /proc/device-tree/cpus/cpu\@0/fsl\,soc-operating-points | xargs -n3 | grep 900000 | cut -d ' ' -f 3`
if [ $voltage_900m -ne 1250000 ]; then
  echo "900MHz voltage: wrong"
  UpdateRecovDTB "false"
  flagVoltageOK="false"
else
  echo "900MHz voltage: correct"
  UpdateRecovDTB "true"
  flagVoltageOK="true"
fi

#--- for IPSP-949 CPU 900MHz Voltage Problem---End

# update config
python3 ${MYSELF_DIR}/upgradeconfig.py

## for ubidetach newapp
MOUNT_DIR_NEWAPP="/mnt/newapp"
UBI_DEVICE_NUM_NEWAPP="9"

## --------------------------------------------------------
ChkKernelPartition () {
    ## Initialized by error value
    RESULT_NG=1

    DEVICE="${1}"
    DUMP_FILE=/tmp/chkkernelpartition_dump.${$}
    sudo rm -f ${DUMP_FILE}

    BLOCK_SZ=$((128*1024))
    FILE_SZ_KERNEL=$(wc -c "${FW_KERNEL}" | cut -d' ' -f 1)
    FILE_SZ_DTB=$(wc -c "${FW_DTB}" | cut -d' ' -f 1)
    BLKLEN_KERNEL=$(( ( ( ${FILE_SZ_KERNEL} + ${BLOCK_SZ} - 1 ) / ${BLOCK_SZ} ) ))
    BLKLEN_DTB=$(( ( ( ${FILE_SZ_DTB} + ${BLOCK_SZ} - 1 ) / ${BLOCK_SZ} ) ))
    TOREAD_LEN_KERNEL=$(( ${BLKLEN_KERNEL} * ${BLOCK_SZ} ))
    TOREAD_LEN_DTB=$(( ${BLKLEN_DTB} * ${BLOCK_SZ} ))
    OFFSET_OF_DTB_IN_KERNEL_PARTITION="0x00F20000"

    sudo nanddump --bb=skipbad -f "${DUMP_FILE}" "${DEVICE}" -l ${TOREAD_LEN_KERNEL}
    if [ ${?} -ne 0 ]; then
        echo "Failed: dump kernel in ${DEVICE}"
    else
        MD5SUM_RESULT=$(md5sum -b "${DUMP_FILE}" | cut -d' ' -f 1)
        MD5SUM_EXPECTED=$(cat "${KERNEL_BLOCKS_MD5SUM}")
        if [ "X${MD5SUM_RESULT}" != "X${MD5SUM_EXPECTED}" ]; then
            echo "Failed: md5sum of dump kernel in ${DEVICE} does not match expected(${MD5SUM_EXPECTED}) vs result(${MD5SUM_RESULT})"
        else
            sudo rm -f ${DUMP_FILE}
            sudo nanddump --bb=skipbad -f "${DUMP_FILE}" "${DEVICE}" -s ${OFFSET_OF_DTB_IN_KERNEL_PARTITION} -l ${TOREAD_LEN_DTB}
            if [ ${?} -ne 0 ]; then
                echo "Failed: dump dtb in ${DEVICE}"
            else
                MD5SUM_RESULT=$(md5sum -b "${DUMP_FILE}" | cut -d' ' -f 1)
                MD5SUM_EXPECTED=$(cat "${DTB_BLOCKS_MD5SUM}")
                if [ "X${MD5SUM_RESULT}" != "X${MD5SUM_EXPECTED}" ]; then
                    echo "Failed: md5sum of dump dtb in ${DEVICE} does not match expected(${MD5SUM_EXPECTED}) vs result(${MD5SUM_RESULT})"
                else
                    RESULT_NG=0
                    echo "Success: md5sum of dump ${DEVICE} matches"
                fi
            fi

        fi
    fi

    sudo rm -f ${DUMP_FILE}
}

## --------------------------------------------------------
touch ${CANCEL_NEW_PLAY}

## Do erase & write status MTD boot flag as "fwupdating"
sudo flash_erase /dev/mtd3 0 8
  if [ ${?} -ne 0 ]; then exit 1; fi
sudo nandwrite -p /dev/mtd3 -s 0x0 ${STATUS_MTD_FILE_FWUPDATING}
  if [ ${?} -ne 0 ]; then exit 2; fi

## make a log of version before update
ver_before=/etc/toa/version/4gpa_version.json
dst_ver=/mnt/data/log/version_before_update.json
if [ -e $ver_before ]; then
  cp $ver_before $dst_ver
else 
  echo '' > $dst_ver
fi

##--------
sleep 3
##--------

## Do erase & write kernel partition
sudo flash_erase /dev/mtd4 0 0
  if [ ${?} -ne 0 ]; then exit 4; fi
sudo nandwrite -p /dev/mtd4 ${FW_KERNEL}
  if [ ${?} -ne 0 ]; then exit 5; fi
sudo nandwrite -p /dev/mtd4 -s 0xf20000 ${FW_DTB}
  if [ ${?} -ne 0 ]; then exit 6; fi

##--------
sleep 3
##--------

## Dump kernel partition and check md5sum
##echo "Seconds:${SECONDS}"
ChkKernelPartition /dev/mtd4
  if [ ${RESULT_NG} -ne 0 ]; then exit 7; fi
##echo "Seconds:${SECONDS}"

##--------
sleep 1
##--------

## Do erase & write status MTD kernel chksum
sudo flash_erase /dev/mtd3 0x00100000 8
  if [ ${?} -ne 0 ]; then exit 8; fi
sudo nandwrite -p /dev/mtd3 -s 0x00100000 ${STATUS_MTD_KERNEL_CHKSUM}
  if [ ${?} -ne 0 ]; then exit 9; fi

##-----------------------------------------------------------
## Unmount newapp ubifs.
if [ -e ${MOUNT_DIR_NEWAPP} ]; then
	sudo killall gst-launch-1.0
	sudo umount "${MOUNT_DIR_NEWAPP}"
	sudo ubidetach /dev/ubi_ctrl -m "${UBI_DEVICE_NUM_NEWAPP}"
fi
##-----------------------------------------------------------

##--------
sleep 10
##--------

## Do write & ubiattach & mount & umount update partition
sudo ubiformat -y /dev/mtd8 -f ${FW_UPDATE_UBI}
  if [ ${?} -ne 0 ]; then exit 10; fi
sudo ubiattach /dev/ubi_ctrl -m 8
  if [ ${?} -ne 0 ]; then exit 11; fi
sudo mkdir -p ${MOUNT_DIR}
  if [ ${?} -ne 0 ]; then exit 12; fi
sudo mount -o ro -t ubifs ubi2_0 ${MOUNT_DIR}
  if [ ${?} -ne 0 ]; then exit 13; fi
if [ ! -f "${CHKFILE_SCRIPT}" ]; then
  echo "Error: ${CHKFILE_SCRIPT} is not found on update partition!!!" 1>&2
  exit 14
fi
sh "${CHKFILE_SCRIPT}" "${MOUNT_DIR}"
  RESULT_OF_CHKFILE=${?}
  if [ ${RESULT_OF_CHKFILE} -ne 0 ]; then exit ${RESULT_OF_CHKFILE}; fi
sudo umount ${MOUNT_DIR}
  if [ ${?} -ne 0 ]; then exit 15; fi
sudo ubidetach /dev/ubi_ctrl -m 8
  if [ ${?} -ne 0 ]; then exit 16; fi

##--------
sleep 10
##--------

# Update flag for RAM/ROM check only when 900MHz voltage is NG
if [ X$flagVoltageOK = X"false" ]; then
  echo "UPDATE_FLAG_first_boot_after_update" > /mnt/data/log/UPDATE_FLAG
else
  sudo rm -f /mnt/data/log/UPDATE_FLAG
fi

## Do erase & write status MTD boot flag as "normal"
sudo flash_erase /dev/mtd3 0 8
  if [ ${?} -ne 0 ]; then exit 17; fi
sudo nandwrite -p /dev/mtd3 -s 0x0 ${STATUS_MTD_FILE_NORMAL}
  if [ ${?} -ne 0 ]; then exit 18; fi

##--------
sleep 3
##--------

echo "Seconds:${SECONDS}"

} 2>&1 | tee ${TMP_LOG_FILE}
sync; sync; sync

#--- for DeBuG only ---
##sudo cp ${TMP_LOG_FILE} /dev/console
##sleep 10 ## because /dev/console is very slow.
#--- for DeBuG only ---

exit 0
