#!/usr/bin/python3
# -*- coding:utf-8 -*-
# upgradeconfig.py は updater の中から実行される
import os
import sys
import json
import re

fileVersion = '/etc/toa/version/4gpa_version.json'

def readFileVersion(filePath):
    try:
        with open(filePath, "r") as fin:
            data = json.load(fin)
            return data["version_for_developer"]
    except:
        return None

def GetCurrentFirmware():
    return readFileVersion(fileVersion)

def SplitVersion(version):
    match = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)\.(\d+)',version)
    if match:
        return ( int(match.group(1)), int(match.group(2)), int(match.group(3)), int(match.group(4)), int(match.group(5)) )
    return None

def CompareVersion(targetVer, baseVer):
    for i in range(0, 5):
        if baseVer[i] < targetVer[i]:
            return 1
        elif baseVer[i] > targetVer[i]:
            return -1
    return 0

def DoesNeedUpdateJSON(baseVer):
    curVer = GetCurrentFirmware()

    if curVer is None:
        return False
    else:
        print("now: ", curVer)
        curVer = SplitVersion(curVer)

        # ver 1.2.3.0.1 より古い FW からの update の場合は設定ファイルを更新する。
        if CompareVersion(curVer, baseVer) < 0:
            print("current version is older than base version")
            return True

    return False

def UpdateUserConfig():
    userJson = '/mnt/data/config/Config_User.json'

    try:
        labelAdminUser = "adm_namein"
        labelAdminPass = "adm_oldpass"
        defaultAdmin   = "admin"
        defaultPass    = "guest"

        updated = False
        data = None

        # adm_namein  : "uc4sc615admin"      => "admin"
        # adm_oldpass : "uc4sc615adminguest" => "guest"
        with open(userJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            jsonData[labelAdminUser] = defaultAdmin
            jsonData[labelAdminPass] = defaultPass
            updated = True
            print("needs update")

        if updated:
            with open(userJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", userJson)
    except:
        pass

def UpdateOnvifConfig():
    onvifJson = '/mnt/data/config/Config_Onvif.json'
    try:
        labelAdminUser = "au_onvifuser"
        labelAdminPass = "au_onvifpass"
        defaultAdmin   = "admin"
        defaultPass    = "guest"

        updated = False
        data = None

        # au_onvifuser : "admin" => "admin"
        # au_onvifpass : "admin" => "guest"
        with open(onvifJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            jsonData[labelAdminUser] = defaultAdmin
            jsonData[labelAdminPass] = defaultPass
            updated = True
            print("needs update")

        if updated:
            with open(onvifJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", userJson)
    except:
        pass


def UpdateGPIOConfig():
    onvifJson = '/mnt/data/config/Config_GPIO.json'
    try:
        labelGPIIn1     = "gpi_inact1"
        labelGPIOUT1    = "gpi_outact1"
        defaultGPIIn1   = 1
        defaultGPIOUT1  = 1

        updated = False
        data = None

        # gpi_inact1  : 0 => 1
        # gpi_outact1 : 0 => 1
        with open(onvifJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            jsonData[labelGPIIn1]  = defaultGPIIn1
            jsonData[labelGPIOUT1] = defaultGPIOUT1
            updated = True
            print("needs update")

        if updated:
            with open(onvifJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", userJson)
    except:
        pass

def UpdatePatternConfig():
    onvifJson = '/mnt/data/config/Config_Pattern.json'
    try:
        labelOptype     = "pat_patoptype"
        valueMediaPlayback = 1
        valueSIPCall       = 2

        updated = False
        data = None

        with open(onvifJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            for i in range(1, 20+1):
                label = labelOptype + str(i)
                jsonData[label]  = valueMediaPlayback

            for i in range(98, 100+1):
                label = labelOptype + str(i)
                jsonData[label]  = valueSIPCall
            updated = True
            print("needs update")

        if updated:
            with open(onvifJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", userJson)
    except:
        pass


def UpdateSIPConfig():
    sipJson = '/mnt/data/config/Config_SIP.json'
    try:
        labelSipSerPort     = "sip_serport"
        defaultSipSerPort   = 5060

        updated = False
        data = None

        # sip_serport  : 2060 => 5060
        with open(sipJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            jsonData[labelSipSerPort]  = defaultSipSerPort
            updated = True
            print("needs update")

        if updated:
            with open(sipJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", sipJson)
    except:
        pass


def UpdateAudioConfig():
    audioJson = '/mnt/data/config/Config_Audio.json'
    try:
        labelMicMute     = "mic_mute"

        updated = False
        data = None

        # mic_mute  : 0 => 1
        with open(audioJson, "r") as fin:
            data = json.load(fin)

            jsonData = data[0]
            jsonData[labelMicMute]  = 1
            updated = True
            print("needs update")

        if updated:
            with open(audioJson, "w") as fout:
                json.dump(data, fout, indent=True, sort_keys=True)
                print("updated: ", audioJson)
    except:
        pass

if __name__ == "__main__":
    # Config_User.json
    baseVer = (1, 2, 3, 0, 1)
    if DoesNeedUpdateJSON(baseVer):
        UpdateUserConfig()
    else:
        print("nothing to do")

    # Config_Onvif.json
    baseVer = (1, 2, 4, 0, 1)
    if DoesNeedUpdateJSON(baseVer):
        UpdateOnvifConfig()
    else:
        print("nothing to do")

    # Config_GPIO.json
    baseVer = (1, 2, 4, 0, 1)
    if DoesNeedUpdateJSON(baseVer):
        UpdateGPIOConfig()
    else:
        print("nothing to do")

    # Config_Pattern.json
    baseVer = (1, 2, 4, 1, 1)
    if DoesNeedUpdateJSON(baseVer):
        UpdatePatternConfig()
    else:
        print("nothing to do")

    # Config_SIP.json
    baseVer = (1, 2, 4, 4, 6)
    if DoesNeedUpdateJSON(baseVer):
        UpdateSIPConfig()
    else:
        print("nothing to do")

    # Config_Audio.json
    baseVer = (1, 4, 1, 0, 0)
    if DoesNeedUpdateJSON(baseVer):
        UpdateAudioConfig()
    else:
        print("nothing to do")

    os.sync()

